#!/bin/bash
###########################################################################
#
# CellDesigner4.4.2.sh - CellDesigner version 4.4.2 for Linux
#
# Copyright (c) 2005-2019 The Systems Biology Institute. All rights reserved.
#
###########################################################################

#JAVA_HOME="$(find /usr/lib/jvm/* -maxdepth 0 -type d -name "*-9*" | head -n1)"

APP_NAME="CellDesigner4.4.2"

JAR_ARGS=-sbwmodule

CD_PREFIX="$(dirname $(readlink -f $0))"

JAVA_HOME="$CD_PREFIX"

MAIN_CLASS=jp.sbi.celldesigner.Application

GUI_LAF="com.sun.java.swing.plaf.gtk.GTKLookAndFeel"
#GUI_LAF="javax.swing.plaf.nimbus.NimbusLookAndFeel"

if [ -f /etc/os-release ]; then
  # freedesktop.org and systemd
  . /etc/os-release
  osname=$NAME
  osversion=$VERSION_ID
elif type lsb_release >/dev/null 2>&1; then
  # linuxbase.org
  osname=$(lsb_release -si)
  osversion=$(lsb_release -sr)
elif [ -f /etc/lsb-release ]; then
  # For some versions of Debian/Ubuntu without lsb_release command
  . /etc/lsb-release
  osname=$DISTRIB_ID
  osversion=$DISTRIB_RELEASE
elif [ -f /etc/debian_version ]; then
  # Older Debian/Ubuntu/etc.
  osname=Debian
  osversion=$(cat /etc/debian_version)
else
  # Fall back to uname, e.g. "Linux <version>", also works for BSD, etc.
  osname=$(uname -s)
  osversion=$(uname -r)
fi

JAVA="$JAVA_HOME/jre/bin/java"
echo $JAVA
VM_ARGS="-XX:+AggressiveHeap -Djava.library.path=. -Dswing.defaultlaf=${GUI_LAF}"

if [ ! -d "${CD_PREFIX}" ]; then
   echo "Error!! ${CD_PREFIX}: No such directory."
   exit 1
fi

cd "${CD_PREFIX}"

CLASS_PATH=$(JARS=(exec/celldesigner.jar exec/autolayout_yobf.jar exec/yObf.jar lib/*.jar); IFS=:; echo "${JARS[*]}")

# to avoid a problem which occurs when compiz enabled.
#export AWT_TOOLKIT="MToolkit"

"${JAVA}" ${VM_ARGS} -cp ${CLASS_PATH} ${MAIN_CLASS} ${JAR_ARGS} 2>&1 | egrep -v "WARN|CRITICAL|^$"

exit 0
